--------------------------------------------------------------------------------
-- Weapon Shield Generator
-- Original Carnage Contest Weapon
-- Script by DC, August 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.v666==nil then cc.v666={} end
cc.v666.antigrav={}
cc.v666.antigrav.gen={}

-- Load & Prepare Ressources
cc.v666.antigrav.sfx_build=loadsfx("buildmetal.ogg")									-- Generator Build Sound
cc.v666.antigrav.sfx_bounce=loadsfx("bounce.wav")										-- Bounce
cc.v666.antigrav.gfx_generator=loadgfx("buildings/v666_AntiGrav.bmp")
setmidhandle(cc.v666.antigrav.gfx_generator)

--------------------------------------------------------------------------------
-- Weapon: Shield Generator
--------------------------------------------------------------------------------

cc.v666.antigrav.id=addweapon("cc.v666.antigrav","Anti-Grav",cc.v666.antigrav.gfx_generator,1) -- Add Weapon (1 use)

function cc.v666.antigrav.draw()														-- Draw
	if weapon_shots<6 then
		-- HUD Positioning
		if weapon_shots==0 then
			hudpositioning(pos_build,cc.v666.antigrav.gfx_generator)
		end
	end
end

function cc.v666.antigrav.attack(attack)												-- Attack
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	if (weapon_shots==0) and (weapon_position==1) then
		weapon_position=0
		-- Use weapon
		useweapon(1)
		weapon_shots=weapon_shots+1
		-- Build Generator
		createobject(cc.v666.antigrav.gen.id,weapon_x,weapon_y)
		-- Sound
		playsound(cc.v666.antigrav.sfx_build)
	end
end

--------------------------------------------------------------------------------
-- Object: Generator
--------------------------------------------------------------------------------

cc.v666.antigrav.gen.id=addobject("cc.v666.antigrav.gen",cc.v666.antigrav.gfx_generator)			-- Add Object
function cc.v666.antigrav.gen.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].health=65
	objects[id].sy=0
	objects[id].agtimer=0
end
function cc.v666.antigrav.gen.draw(id,x,y)
	-- Smoke Particles?
	if objects[id].health<=30 then
		if getframe()%3==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
			particlecolor(25,25,25)
			if getframe()%12==0 then
				particle(p_spark,x-16,y-2+math.random(-4,5))
			end
		end
	else
		if getframe()%9==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
		end		
	end
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1+math.abs(math.sin(id*10+os.clock()*3))*0.1,1+math.abs(math.sin(id*10+os.clock()*3))*0.1)
	setrotation(0)
	-- Draw
	drawimage(cc.v666.antigrav.gfx_generator,x,y)
end
function cc.v666.antigrav.gen.update(id,x,y)
	local plt=playertable(0,0)
	if getframe()==0 then objects[id].agtimer=0 end
	if getframesleft()<1 then objects[id].agtimer=objects[id].agtimer+1 end
	-- Gravity influence on players
		for i=1,#plt do
			if (getframesleft()>0 or objects[id].agtimer<250) and math.abs(getplayerx(plt[i])-x)<16 and getplayery(plt[i])<y and getplayery(plt[i])>y-600 then
				playerpush(plt[i],0,-4*getgravity(),0,0)
				particle(p_flare,x,y)
				particlesize(10,7)
				particlecolor(0,100,255)
			end
		end
	-- Gravity influence on projectiles
		for i=1,#projectiles do
			if (getframesleft()>0 or objects[id].agtimer<250) and projectiles[i].sy~=nil and projectiles[i].x~=nil and math.abs(projectiles[i].x-x)<16 and projectiles[i].y<y and projectiles[i].y>y-600 then
				projectiles[i].sy=projectiles[i].sy-3*getgravity()
				particle(p_flare,x,y)
				particlesize(10,7)
				particlecolor(0,100,255)
			end
		end
	-- Gravity influence on objects
	local olt=objecttable()
		if getframesleft()>0 or objects[id].agtimer<250 then
			for i=1,#olt do
				if math.abs(getobjectx(olt[i])-x)<16 and olt[i]~=id and getobjecty(olt[i])<y and getobjecty(olt[i])>y-600 then
					if objects[olt[i]]~=nil and objects[olt[i]].sy~=nil then
						objects[olt[i]].sy=objects[olt[i]].sy-3*getgravity()
						objectpush(olt[i],0,-3*getgravity())
					end
					particle(p_flare,x,y)
					particlesize(10,7)
					particlecolor(0,100,255)
				end
			end
		end
	-- Gravity influence on speed
	objects[id].sy=objects[id].sy+3*getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.abs(objects[id].sy)/15)
	msuby=objects[id].sy/msubt
	for i=1,msubt,1 do
		y=y+msuby
		-- Collision
		if collision(cc.v666.antigrav.gfx_generator,x,y,1,1,1,-1,id)==1 then
			if (math.abs(objects[id].sy)>0.5) then playsound(cc.v666.antigrav.sfx_bounce) end
			y=y-msuby
			objects[id].sy=-objects[id].sy*0.1
			msuby=-msuby*0.3
		end
		-- Water
		if y>getwatery()-5 then
			-- Effects
			particle(p_waterhit,x,y)
			playsound(sfx_hitwater1)
			-- Remove Generator
			removeobject(id)
			break
		end
	end
	objectposition(id,x,y)
end
function cc.v666.antigrav.gen.damage(id,damage)
	if objects[id].health>0 then 
		objects[id].health=objects[id].health-damage
		if objects[id].health<=0 then
			local x=getobjectx(id)
			local y=getobjecty(id)
			-- Destroy terrain
			terrainexplosion(x,y,30,1)
			-- Crater
			grey=math.random(0,40)
			if math.random(0,1)==1 then
				terrainalphaimage(gfx_crater100,x,y,math.random(6,9)*0.1,grey,grey,grey)
			else
				terrainalphaimage(gfx_crater125,x,y,math.random(6,9)*0.1,grey,grey,grey)
			end
			-- Remove Generator
			removeobject(id)
			-- Cause damage
			arealdamage(x,y,100,50)
		end
	end
end