--------------------------------------------------------------------------------
-- Weapon Shield Generator
-- Original Carnage Contest Weapon
-- Script by DC, August 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.v666==nil then cc.v666={} end
cc.v666.gravcenter={}
cc.v666.gravcenter.gen={}

-- Load & Prepare Ressources
cc.v666.gravcenter.sfx_build=loadsfx("buildmetal.ogg")									-- Generator Build Sound
cc.v666.gravcenter.sfx_bounce=loadsfx("bounce.wav")										-- Bounce
cc.v666.gravcenter.gfx_generator=loadgfx("buildings/v666_grav_center.bmp")
setmidhandle(cc.v666.gravcenter.gfx_generator)

--------------------------------------------------------------------------------
-- Weapon: Shield Generator
--------------------------------------------------------------------------------

cc.v666.gravcenter.id=addweapon("cc.v666.gravcenter","Gravity Center",cc.v666.gravcenter.gfx_generator,0) -- Add Weapon (0 uses)

function cc.v666.gravcenter.draw()														-- Draw
	if weapon_shots<6 then
		-- HUD Positioning
		if weapon_shots==0 then
			hudpositioning(pos_build,cc.v666.gravcenter.gfx_generator,200)
		end
	end
end

function cc.v666.gravcenter.attack(attack)												-- Attack
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	if (weapon_shots==0) and (weapon_position==1) then
		weapon_position=0
		-- Use weapon
		useweapon(0)
		weapon_shots=weapon_shots+1
		-- Build Generator
		createobject(cc.v666.gravcenter.gen.id,weapon_x,weapon_y)
		-- Sound
		playsound(cc.v666.gravcenter.sfx_build)
		-- End Turn
		endturn()
	end
end

--------------------------------------------------------------------------------
-- Object: Generator
--------------------------------------------------------------------------------

cc.v666.gravcenter.gen.id=addobject("cc.v666.gravcenter.gen",cc.v666.gravcenter.gfx_generator)			-- Add Object
function cc.v666.gravcenter.gen.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].health=100
	objects[id].sy=0
	objects[id].agtimer=0
end

function cc.v666.gravcenter.gen.draw(id,x,y)
	-- Smoke Particles?
	if objects[id].health<=30 then
		if getframe()%3==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
			particlecolor(25,25,25)
			if getframe()%12==0 then
				particle(p_spark,x-16,y-2+math.random(-4,5))
			end
		end
	else
		if getframe()%9==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
		end		
	end
	if getframe()%5==0 then
		particle(p_flare,x,y)
		particlesize(9,11)
		particlecolor(255,5,5)
	end
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1+math.abs(math.sin(id*10+os.clock()*3))*0.1,1+math.abs(math.sin(id*10+os.clock()*3))*0.1)
	setrotation(0)
	-- Draw
	drawimage(cc.v666.gravcenter.gfx_generator,x,y)
end

function cc.v666.gravcenter.gen.update(id,x,y)
	local plt=playertable(0,0)
	if getframe()==0 then objects[id].agtimer=0 end
	if getframesleft()<1 then objects[id].agtimer=objects[id].agtimer+1 end
	-- Gravity influence on players
	for i=1,#plt do
		local dx=getplayerx(i)-getobjectx(id)
		local dy=getplayery(i)-getobjecty(id)
		local r=math.sqrt(dx*dx+dy*dy)
		if (getframesleft()>0 or objects[id].agtimer<250) and getplayeraction(plt[i])~=0 then
			playerpush(plt[i],-dx/(r^3)*805,-dy/(r^3)*805,0,0)
		end
	end
	-- Gravity influence on projectiles
	for i=1,#projectiles do
		if projectiles[i].sy~=nil and projectiles[i].sx~=nil then
			local dx=projectiles[i].x-getobjectx(id)
			local dy=projectiles[i].y-getobjecty(id)
			local r=math.sqrt(dx*dx+dy*dy)
			projectiles[i].sx=projectiles[i].sx-(dx/(r^3)*875)
			projectiles[i].sy=projectiles[i].sy-(dy/(r^3)*875)
		end
	end
	-- Gravity influence on objects
	local olt=objecttable()
	for i=1,#olt do
		if getframesleft()>0 and objects[id].agtimer<250 and math.abs(getobjectyspeed(olt[i]))>3 then
			if objects[olt[i]]~=nil and objects[olt[i]].sy~=nil and objects[olt[i]].sx~=nil then
				local dx=getobjectx(olt[i])-getobjectx(id)
				local dy=getobjecty(olt[i])-getobjecty(id)
				local r=math.sqrt(dx*dx+dy*dy)
				objects[olt[i]].sx=objects[olt[i]].sx-(dx/(r^3)*800)
				objects[olt[i]].sy=objects[olt[i]].sy-(dy/(r^3)*800)
			end
		end
	end
end

function cc.v666.gravcenter.gen.damage(id,damage)
	if objects[id].health>0 then 
		objects[id].health=objects[id].health-damage
		if objects[id].health<=0 then
			local x=getobjectx(id)
			local y=getobjecty(id)
			-- Destroy terrain
			terrainexplosion(x,y,30,1)
			-- Crater
			grey=math.random(0,40)
			if math.random(0,1)==1 then
				terrainalphaimage(gfx_crater100,x,y,math.random(6,9)*0.1,grey,grey,grey)
			else
				terrainalphaimage(gfx_crater125,x,y,math.random(6,9)*0.1,grey,grey,grey)
			end
			-- Remove Generator
			removeobject(id)
			-- Cause damage
			arealdamage(x,y,100,50)
		end
	end
end