--------------------------------------------------------------------------------
-- Weapon Shield Generator
-- Original Carnage Contest Weapon
-- Script by DC, August 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.v666==nil then cc.v666={} end
cc.v666.hgravgen={}
cc.v666.hgravgen.gen={}

-- Load & Prepare Ressources
cc.v666.hgravgen.sfx_build=loadsfx("buildmetal.ogg")									-- Generator Build Sound
cc.v666.hgravgen.sfx_bounce=loadsfx("bounce.wav")										-- Bounce
cc.v666.hgravgen.gfx_generator=loadgfx("buildings/v666_grav_gen.bmp")
setmidhandle(cc.v666.hgravgen.gfx_generator)

--------------------------------------------------------------------------------
-- Weapon: Shield Generator
--------------------------------------------------------------------------------

cc.v666.hgravgen.id=addweapon("cc.v666.hgravgen","High gravity Generator",cc.v666.hgravgen.gfx_generator,0) -- Add Weapon (0 uses)

function cc.v666.hgravgen.draw()														-- Draw
	if weapon_shots<6 then
		-- HUD Positioning
		if weapon_shots==0 then
			hudpositioning(pos_build,cc.v666.hgravgen.gfx_generator)
		end
	end
end

function cc.v666.hgravgen.attack(attack)												-- Attack
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	if (weapon_shots==0) and (weapon_position==1) then
		weapon_position=0
		-- Use weapon
		useweapon(0)
		weapon_shots=weapon_shots+1
		-- Build Generator
		createobject(cc.v666.hgravgen.gen.id,weapon_x,weapon_y)
		-- Sound
		playsound(cc.v666.hgravgen.sfx_build)
		-- End Turn
		endturn()
	end
end

--------------------------------------------------------------------------------
-- Object: Generator
--------------------------------------------------------------------------------

cc.v666.hgravgen.gen.id=addobject("cc.v666.hgravgen.gen",cc.v666.hgravgen.gfx_generator)			-- Add Object
function cc.v666.hgravgen.gen.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].health=65
	objects[id].sy=0
	objects[id].agtimer=0
end
function cc.v666.hgravgen.gen.draw(id,x,y)
	-- Smoke Particles?
	if objects[id].health<=30 then
		if getframe()%3==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
			particlecolor(25,25,25)
			if getframe()%12==0 then
				particle(p_spark,x-16,y-2+math.random(-4,5))
			end
		end
	else
		if getframe()%9==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
		end		
	end
	if getframe()%5==0 then
		particle(p_flare,x,y)
		particlesize(10,10)
		particlecolor(255,0,0)
	end
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1+math.abs(math.sin(id*10+os.clock()*3))*0.1,1+math.abs(math.sin(id*10+os.clock()*3))*0.1)
	setrotation(0)
	-- Draw
	drawimage(cc.v666.hgravgen.gfx_generator,x,y)
end
function cc.v666.hgravgen.gen.update(id,x,y)
	local plt=playertable(0,0)
	if getframe()==0 then objects[id].agtimer=0 end
	if getframesleft()<1 then objects[id].agtimer=objects[id].agtimer+1 end
	-- Gravity influence on players
		for i=1,#plt do
			if getplayeraction(plt[i])~=0 then
				playerpush(plt[i],0,getgravity(),0,0)
			end
		end
	-- Gravity influence on projectiles
		for i=1,#projectiles do
			if projectiles[i].sy~=nil then
				projectiles[i].sy=projectiles[i].sy+getgravity()
			end
		end
	-- Gravity influence on objects
	local olt=objecttable()
		for i=1,#olt do
			if getframesleft()>0 and objects[id].agtimer<250 and math.abs(getobjectyspeed(olt[i]))>3 then
				if objects[olt[i]]~=nil and objects[olt[i]].sy~=nil then
					objects[olt[i]].sy=objects[olt[i]].sy+getgravity()
				end
			end
		end
	-- Gravity influence on speed
	objects[id].sy=objects[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.abs(objects[id].sy)/15)
	msuby=objects[id].sy/msubt
	for i=1,msubt,1 do
		y=y+msuby
		-- Collision
		if collision(cc.v666.hgravgen.gfx_generator,x,y,1,1,1,-1,id)==1 then
			if (math.abs(objects[id].sy)>0.5) then playsound(cc.v666.hgravgen.sfx_bounce) end
			y=y-msuby
			objects[id].sy=-objects[id].sy*0.3
			msuby=-msuby*0.3
		end
		-- Water
		if y>getwatery()-5 then
			-- Effects
			particle(p_waterhit,x,y)
			playsound(sfx_hitwater1)
			-- Remove Generator
			removeobject(id)
			break
		end
	end
	objectposition(id,x,y)
end
function cc.v666.hgravgen.gen.damage(id,damage)
	if objects[id].health>0 then 
		objects[id].health=objects[id].health-damage
		if objects[id].health<=0 then
			local x=getobjectx(id)
			local y=getobjecty(id)
			-- Destroy terrain
			terrainexplosion(x,y,30,1)
			-- Crater
			grey=math.random(0,40)
			if math.random(0,1)==1 then
				terrainalphaimage(gfx_crater100,x,y,math.random(6,9)*0.1,grey,grey,grey)
			else
				terrainalphaimage(gfx_crater125,x,y,math.random(6,9)*0.1,grey,grey,grey)
			end
			-- Remove Generator
			removeobject(id)
			-- Cause damage
			arealdamage(x,y,100,50)
		end
	end
end